# -*- coding: utf-8 -*-

import netCDF4 as nc
from netCDF4 import Dataset # to work with NetCDF files
import numpy as np
import datetime as dt
import sys
from collections import OrderedDict

# Set default print options for 
np.set_printoptions(edgeitems=3,infstr='inf',
linewidth=75, nanstr='nan', precision=8,
suppress=False, threshold=1000, formatter=None)

### CONSTANTS ###
cropnumber=8
missingval = 1.e+20
xdimension_name='rlon'
ydimension_name='rlat'
xdimension_reg_name='lon'
ydimension_reg_name='lat'
compress=True
ref_date=dt.datetime(1949,12,1,00,00,00)



# ******************************************************************************
def rotpole2regular(lon,lat,varval,coord_1di,interp_method,xi,yi, rotpolemask):

  """ Interpolates a time-varying field from the native WRF grid mesh (rotated pole)
  to a regular lat-lon grid mesh.
  The regular lat-lon grid mesh is defined by coordinates in the a netcdf file in globar variables 
  gvar.output_regular_grid.

  Input: 
  lon: longitude of the native grid 
  lat: latitude of the native grid
  varval: 2-D or 3-D variable to interpolate 
  coord_1di: output grid mesh
  inter_method: interpolation method (default is nearest but linear or cspline can also be used)
  rotpolemask: Mask of rotated pole grid on the regular grid

 
  Output: a 2-D or 3-D variable with data in the regular grid mesh.

  Author: Alejanro Di Luca
  Created: 29/09/2015
  Last Modification: 29/09/2015

  HISTORY
  ==========
  Sep 29 2015 => Written by Alejandro Di Luca, CCRC, UNSW
  Feb 04 2016 => Bug fix for zero values; Roman Olson, CCRC, UNSW.

  """
    
  import scipy.interpolate
  import copy as cp

  if varval.ndim==3:
    nt = varval.shape[0]
    rotpolemask_full = np.tile(rotpolemask, (nt, 1, 1))
  else:
    rotpolemask_full = rotpolemask
     

  if varval.ndim==3:
    new_var=np.ones((varval.shape[0],xi.shape[0],xi.shape[1]))
    for ii,tim in enumerate(range(0,varval.shape[0])):
      new_var[ii,:,:]=scipy.interpolate.griddata((lon.flatten(),lat.flatten()),\
                                                   varval[ii,:,:].flatten(),coord_1di.T,method=interp_method).reshape(yi.shape)

  if varval.ndim==2:
    new_var=np.ones((xi.shape[0],xi.shape[1]))
    new_var[:,:]=scipy.interpolate.griddata((lon.flatten(),lat.flatten()),\
                                                   varval[:,:].flatten(),coord_1di.T,method=interp_method).reshape(yi.shape)

  if varval.ndim==1:
        sys.exit('Input variable has only one dimension but at least two dimensions are needed.')

  if varval.ndim==4:
        sys.exit('Input variable has too many diemnsions: four dimensions.')

  # replace extrapolated values by missing values
  temp=cp.copy(new_var)
  assert(temp.shape == rotpolemask_full.shape)
  temp[rotpolemask_full==0.0] = missingval # const.missingval -> missingval

  return temp



##### RUN THE NATIVE TO REGULAR INTERPOLATION #####
"""

def run_interp():
    
Description:
------------

Calls rotpole2regular() above to interpolate a file on a rotated grid to a regular
grid with 1D X,Y coordinates. 

Requires the constants starting at line 8 above to be defined. 

Depends upon the packages listed at the top of the file.

Requires a template file 'CORDEX_AUS-44i_regular_grid8.nc' in order to run the interpolation.

Currenly only works for d01.

Inputs:
-------

1. file_freq: one of 'mon' or 'day'
2. year_range: a list containing the year ranges on each input file: 
    e.g. for daily WRF: year_range = ['19790101-19801231', '19810101-19851231', '19860101-19901231', '19910101-19951231', '19960101-20001231', '20010101-20051231', 
                                      '20060101-20101231']
   or e.g. for daily COSMO-CLM: ['19810101-19851231', '19860101-19901231', '19910101-19951231', '19960101-20001231', '20010101-20051231', '20060101-20101231']

3. time_bounds: Boolean
4. variables: a string or list of however many variables you want to process, e.g. variables = 'rlus' or variables = ['rlus','tasmax']
5. rcm_id: The specific id of the RCM you want to process e.g. 'UNSW-WRF360L'. 
6. file_dir: The code assumes you have your input data in a directory like the following:
   
   E:/ERA-Int/Annette/ERA-Int/AUS-44/UNSW-WRF360L/day/rlus
   
   You just need to pass it the base directory containing the data, i.e. the first part of the above: 
       
   'E:/ERA-Int/Annette/ERA-Int'

NOTE1: run_interp() expects the output directory to already exist in its entirety,
e.g. if you are processing rlus, it expects an empty directory (where the function
automatically changes 'AUS-44' to 'AUS-44i'):
    
E:/ERA-Int/Annette/ERA-Int/AUS-44i/UNSW-WRF360L/day/rlus
    
NOTE2: Currently only handles one rcm run at a time (so you'd run it for UNSW-WRF360J, then for UNSW-WRF360K, etc).
An improvement would be to loop over as many rcm runs as required.


######################## Example call to run_interp() ########################

run_interp(file_freq='day', year_range=['19790101-19801231', '19810101-19851231'],
           time_bounds=True, variables=['rlus','hfls'], rcm_id='UNSW-WRF360L', 
           file_dir='E:/ERA-Int/Annette/ERA-Int')

Assuming you called run_interp() using the above example, and all inputs were
present and correct, you would end up with the files listed below:
  
E:/ERA-Int/Annette/ERA-Int/AUS-44i/UNSW-WRF360L/day/rlus

rlus_AUS-44i_ECMWF-ERAINT_evaluation_r1i1p1_UNSW-WRF360L_v1_day_19790101-19801231.nc
rlus_AUS-44i_ECMWF-ERAINT_evaluation_r1i1p1_UNSW-WRF360L_v1_day_19810101-19851231.nc

and:

E:/ERA-Int/Annette/ERA-Int/AUS-44i/UNSW-WRF360L/day/hfls

hfls_AUS-44i_ECMWF-ERAINT_evaluation_r1i1p1_UNSW-WRF360L_v1_day_19790101-19801231.nc
hfls_AUS-44i_ECMWF-ERAINT_evaluation_r1i1p1_UNSW-WRF360L_v1_day_19810101-19851231.nc

"""


def run_interp(file_freq, year_range, time_bounds, variables, rcm_id, file_dir):

   att = OrderedDict()
   att['_FillValue'] = missingval
   att['missing_value'] = missingval
   
   for varname in variables:
   	for interval in year_range:
   		print('PROCESSING: ', varname)
   		
   		grid_type = 'AUS-44'			
   		
   		# ERA-Interim WRF 
   		native_file_name = '%s_%s_ECMWF-ERAINT_evaluation_r1i1p1_%s_v1_%s_%s.nc' %(varname, grid_type, rcm_id, file_freq, interval)
   		native_file = file_dir + '/%s/%s/%s/%s/' %(grid_type, rcm_id, file_freq, varname) + native_file_name
   		
   		""" Get the variable """
   		var_nat = Dataset(native_file, mode='r')
   		time = var_nat.variables['time'][:]
   		time_units = var_nat.variables['time'].units
   		time_bnds = var_nat.variables['time_bnds'][:]
   		varval = var_nat.variables[varname][:]
   		# NetCDF global attributes    
   		global_atts = {a:var_nat.getncattr(a) for a in var_nat.ncattrs()}
   		
   		# GDV: Added this code to handle time variables that are required
   		#time = nc.num2date(var_nat.variables['time'][:],units=var_nat.variables['time'].units)
   		#mvar,mtime     = compute_monthly(varval,time,stat)
   		#time = date2hours(mtime,ref_date)
   		
   		var_nat.close()
   		varval.shape
   		
   		fin1 = Dataset(native_file, mode='r')
   		
   		temp=fin1.variables['lon']
   		if temp.ndim==2:
   		  lon=np.squeeze(fin1.variables['lon'][:]) # Getting longitude
   		  lat=np.squeeze(fin1.variables['lat'][:]) # Getting latitude
   		if temp.ndim==3:
   		  lon=np.squeeze(fin1.variables['lon'][0,:,:]) # Getting longitude
   		  lat=np.squeeze(fin1.variables['lat'][0,:,:]) # Getting latitude
   		fin1.close()
   		 
   		#####sch_info=read_schemes(fileref_att) # NOT IMPLEMENTED
   		
   		# Transform longitudes to the 0-360 range for the interpolation
   		# purposes
   		lon[lon < 0] = 360 + lon[lon < 0]
   		
   		# Crop the coordinates for stats calculation -- THINK THIS SHOULD BE TRUE?
   		stats = False
   		if stats: 
   		   y_lat         = lat.shape[0] 
   		   x_lon         = lon.shape[1] 
   		   slatindex     = slonindex = cropnumber 
   		   elatindex     = y_lat - cropnumber
   		   elonindex     = x_lon - cropnumber
   		   lon_pt        = lon[slatindex:elatindex, slonindex:elonindex]
   		   lat_pt        = lat[slatindex:elatindex, slonindex:elonindex]
   		else:
   		   lat_pt = lat
   		   lon_pt = lon
   		   
   		# **********************************************************************
   		# Read the output grid. These lons are already 0-360 degrees.
   		#out_grid = 'I:/UNSW/CCRC/docs/romanolson-cordex_postprocessing_scripts/CORDEX_postprocess/Info_files/CORDEX_AUS-44i_regular_grid8.nc'
   		out_grid = '/home/561/azh561/cordex_australasia/scripts/geo_em_files/CORDEX_AUS-44i_regular_grid8.nc'
   		fin1=Dataset(out_grid,mode='r')
   		loni=fin1.variables['lon'][:]
   		lati=fin1.variables['lat'][:]
   		rotpolemask=fin1.variables['rotpole_mask'][:]
   		fin1.close()
   		xi,yi = np.meshgrid(loni,lati)
   		coord_1di=np.vstack([xi.flatten(),yi.flatten()])
   		interp_method='nearest'
   		
   		
   		# **********************************************************************
   		# INTERPOLATE THE VARIABLE FROM THE CURVILINEAR GRID TO THE REGULAR
   		# LAT-LON GRID 
   		print('  --> INTERPOLATING IN THE REGULAR GRID ')
   		new_var=rotpole2regular(lon_pt,lat_pt,varval,coord_1di,interp_method,xi,yi,
   		   rotpolemask)
   		"""
   		lon_pt.shape
   		lat_pt.shape
   		varval.shape
   		coord_1di.shape
   		xi.shape
   		yi.shape
   		"""
   		
   		# Transform longitudes to -180 to 180 range
   		lon[lon >= 180.0] = lon[lon >= 180.0] - 360.0
   		loni[loni >= 180.0] = loni[loni >= 180.0] - 360.0
   			
   		#**********************************************************************
   		# CREATING NETCDF FILE
   		# Create output file
   		out_file_name = native_file_name
   		out_grid_type = 'AUS-44i'
   		out_file_name = out_file_name.replace("AUS-44",out_grid_type)
   
   		# ERA-Int WRF
   		file_out = file_dir + '/%s/%s/%s/%s/' %(out_grid_type, rcm_id, file_freq, varname) + out_file_name          
   											
   		# File out for testing
   		#file_out = 'G:/ERA-Int/Testing/COSMO-CLM/cclm_test.nc'
   		
   		fout=Dataset(file_out,mode='w', format='NETCDF4_CLASSIC')
   		  
   		# ------------------------
   		# Create dimensions
   		print('   CREATING AND WRITING DIMENSIONS: ')
   		print('              TIME, X, Y(, TIME_BNDS)')
   		#if file_freq!='fx':
   		fout.createDimension(xdimension_reg_name,new_var.shape[2])
   		fout.createDimension(ydimension_reg_name,new_var.shape[1])
   		fout.createDimension('time',None)
   		fout.createDimension('bnds', 2)
   		#else:
   		#  fout.createDimension(const.xdimension_reg_name,new_var.shape[1])
   		#  fout.createDimension(const.ydimension_reg_name,new_var.shape[0])
   		
   		# ------------------------
   		# Create and assign values to variables
   		print( "\n")
   		print('   CREATING AND WRITING VARIABLES:')
   		
   		# VARIABLE: longitude
   		print('  ---   LONGITUDE VARIABLE CREATED ')
   		varout=fout.createVariable('lon','f8',[xdimension_reg_name], zlib=compress, 
   		 complevel=5)
   		varout[:]=loni[:]
   		setattr(varout, 'standard_name','longitude')
   		setattr(varout, 'long_name','Longitude')
   		setattr(varout, 'units','degrees_east')
   		setattr(varout, 'axis','X')
   		
   		# VARIABLE: latitude
   		print('    ---   LATITUDE VARIABLE CREATED ' )
   		varout=fout.createVariable('lat','f8',[ydimension_reg_name], zlib=compress,
   		 complevel=5)
   		varout[:]=lati[:]
   		setattr(varout, 'standard_name','latitude')
   		setattr(varout, 'long_name','Latitude')
   		setattr(varout, 'units','degrees_north')
   		setattr(varout, 'axis','Y')
   		varatt={}
   		
   		# VARIABLE: time in days
   		print('    ---   TIME VARIABLE CREATED ' )
   		varout=fout.createVariable('time','f8',['time'], zlib=compress,
   		   complevel=5)
   		varout[:]=np.array(time[:])#/24.0
   		setattr(varout, 'standard_name','time')
   		setattr(varout, 'long_name','time')
   		setattr(varout, 'bounds','time_bnds')
   		#setattr(varout, 'units','days since %s' %(ref_date.strftime("%Y-%m-%d %H:%M:%S")))
   		setattr(varout, 'units',time_units)
   		setattr(varout, 'calendar','standard')
   		
   		# VARIABLE: time_bnds in days
   		if time_bounds==True:
   		  print('  ---   TIME_BNDS VARIABLE CREATED ')
   		  varout=fout.createVariable('time_bnds','f8',['time', 'bnds'], 
   			 zlib=compress, complevel=5)
   		  varout[:]=np.array(time_bnds[:])#/24.0
   		  #setattr(varout, 'units','days since %s' %(ref_date.strftime("%Y-%m-%d %H:%M:%S")))
   		  setattr(varout, 'units',time_units)
   		  setattr(varout, 'calendar','standard')
   		
   		# VARIABLE: variable
   		print('    ---   ',varname, ' VARIABLE CREATED ' )
   		varout=fout.createVariable(varname,'f',['time', ydimension_reg_name, xdimension_reg_name], 
   		   fill_value=att['_FillValue'], zlib=compress, complevel=5)
   		varout[:]=new_var[:]
   		for att in varatt.keys():
   		  if att!='_FillValue' and att!='grid_mapping':
   			  if att[att]!=None:
   				  setattr(varout, att, att[att])
   		
   		# WRITE GLOBAL ATTRIBUTES 
   		print('\n','CREATING AND WRITING GLOBAL ATTRIBUTES:')
   		fout.setncatts(global_atts)
   		#gblatt = get_globatt(gvars.GCMModelName,gvars.RCMModelName,gvars.institution,sch_info,file_freq, gvars.RCMVersionID, gvars.CMIP5EnsembleMember,regular=True)
   		#for att in gblatt.keys():
   		#  setattr(fout, att, gblatt[att])
   		fout.close()
   		print('  ===> FILE: ', file_out)
   			
   		print('     ------------ FILE SUCCESSFULLY INTERPOLATED! ------------ ')
		
run_interp(file_freq='day', year_range=['19800101-19801231', '19810101-19851231', '19860101-19901231', '19910101-19951231', '19960101-20001231', '20010101-20051231', '20060101-20101231'],
           #time_bounds=True, variables=['rlus','rlds','rsds','rsus','mrso','hfls','hfss','tas','tasmax','tasmin','ts','pr'], rcm_id='CLMcom-CCLM4-8-17-CLM3-5', 
           time_bounds=True, variables=['tasmax','tasmin','ts','pr'], rcm_id='CLMcom-CCLM4-8-17-CLM3-5', 
           file_dir='/g/data3/w97/azh561/cordex_australasia/CCLM4-8/')
