#!/bin/bash
#PBS -P w97
#PBS -q express
#PBS -l walltime=24:00:00,mem=12GB
#PBS -l wd

module purge
module load pbs dot
module load netcdf/4.4.1.1
module load nco
module load cdo/1.8.2

# This script prepares the MERRA2 reanalysis for the interpolated lat/lon CORDEX domain
# to enable direct comparison between MERRA2 and WRF output
#
# Created: Wed Apr  4 10:57:38 AEST 2018 by Annette L Hirsch @ CLEX, UNSW a.hirsch@unsw.edu.au

# Program Start

# USER INPUT ARGUMENTS

RES="AUS-44i"
INPUT_DIR="/g/data1/rr7/MERRA2/raw/"
REGRID_DIR="/g/data3/w97/azh561/cordex_australasia/MERRA2_on_CORDEX_${RES}/"
NATIVE_DIR="/g/data3/w97/azh561/cordex_australasia/MERRA2/"
RUN_DIR="/short/w97/azh561/"
TMP_DIR="${RUN_DIR}/tmp/"
mkdir -p ${TMP_DIR}

STYEAR=$1
FIYEAR=$2
VARIABLES=$3 

cat > targetgrid << EOF
gridtype=lonlat
xsize=238
ysize=133
yfirst=-53.25
yinc=0.5
xfirst=88.75
xinc=0.5
EOF

echo 'Start time: '`date`

for vv in ${VARIABLES[@]}; do

  mkdir -p ${REGRID_DIR}/${vv}
  mkdir -p ${NATIVE_DIR}/${vv}

  YEAR=${STYEAR}

  if [[ ${vv} =~ ^(tmax|tmin|T2M)$ ]]; then 
    INDIR="${INPUT_DIR}/M2T1NXSLV.5.12.4/"
    FPREFIX="MERRA2_100.tavg1_2d_slv_Nx."
  elif [[ ${vv} =~ ^(TS|EMIS|ALBEDO|LWGEM|LWGAB|LWGNT|SWGDN|SWGNT|CLDTOT)$ ]]; then 
    INDIR="${INPUT_DIR}/M2T1NXRAD.5.12.4/"
    FPREFIX="MERRA2_100.tavg1_2d_rad_Nx."
  elif [[ ${vv} =~ ^(EFLUX|HFLUX|EVAP|PRECTOT)$ ]]; then 
    INDIR="${INPUT_DIR}/M2T1NXFLX.5.12.4/"
    FPREFIX="MERRA2_100.tavg1_2d_flx_Nx."
  fi

  # Loop through the years
  while [ ${YEAR} -lt $((${FIYEAR}+1)) ]; do

    if [ ! -e "${NATIVE_DIR}/${vv}/MERRA2_${vv}_${YEAR}.nc" ]; then

      echo "File ${NATIVE_DIR}/${vv}/MERRA2_${vv}_${YEAR}.nc does not exist"
      echo "Calculating daily values for each year"

      cd ${INDIR}/${YEAR}
      # Loop through the months and collate 
      MONLIST=`ls -1`
      for mm in ${MONLIST[@]}; do
      
        cd ${mm}
        # Loop through the daily files to calculate daily values
        DAYLIST=`ls -1`
        for dd in ${DAYLIST[@]}; do
          INFILE=${dd}
          day=${dd: -6:2}

          if [ ${vv} == "tmax" ]; then
            cdo -daymax ${INFILE} "${TMP_DIR}/dd_${vv}_${INFILE}"
            ncks -v time,time_bnds,T2M "${TMP_DIR}/dd_${vv}_${INFILE}" "${TMP_DIR}/${vv}_${INFILE}"
            ncrename -v T2M,tmax "${TMP_DIR}/${vv}_${INFILE}"
            ncatted -a long_name,tmax,o,c,"Daily Maximum Temperature" "${TMP_DIR}/${vv}_${INFILE}"
          elif [ ${vv} == "tmin" ]; then
            cdo -daymin ${INFILE} "${TMP_DIR}/dd_${vv}_${INFILE}"
            ncks -v time,time_bnds,T2M "${TMP_DIR}/dd_${vv}_${INFILE}" "${TMP_DIR}/${vv}_${INFILE}"
            ncrename -v T2M,tmin "${TMP_DIR}/${vv}_${INFILE}"
            ncatted -a long_name,tmin,o,c,"Daily Minimum Temperature" "${TMP_DIR}/${vv}_${INFILE}"
          else
            cdo -daymean ${INFILE} "${TMP_DIR}/dd_${vv}_${INFILE}"
            ncks -v time,time_bnds,${vv} "${TMP_DIR}/dd_${vv}_${INFILE}" "${TMP_DIR}/${vv}_${INFILE}"
          fi
          rm "${TMP_DIR}/dd_${vv}_${INFILE}"

          # Redefine the time variable
          dateyear=`date -d ${YEAR}-${mm}-${day} +%s`
          refdate=`date -d 1950-01-01 +%s`
          OFFSET=$(((${dateyear} - ${refdate})/86400))

          ncap2 -O -s "time[time]=$((1+${OFFSET}))" "${TMP_DIR}/${vv}_${INFILE}" "${TMP_DIR}/time_${vv}_${INFILE}" 
          ncatted -O -a calendar,time,c,c,"standard" "${TMP_DIR}/time_${vv}_${INFILE}"
          ncatted -a long_name,time,o,c,"time" "${TMP_DIR}/time_${vv}_${INFILE}"
          ncatted -a units,time,o,c,"days since 1950-01-01 00:00:00" "${TMP_DIR}/time_${vv}_${INFILE}" 
          rm "${TMP_DIR}/${vv}_${INFILE}"

        done # for dd in ${DAYLIST[@]}; do
        cd ../

      done # for mm in ${MONLIST[@]}; do

      # Combine all daily files into one for the year and then clean-up
      TIME_FILES=`ls -1 ${TMP_DIR}/time_${vv}_*${YEAR}*`
      ncrcat ${TIME_FILES} "${NATIVE_DIR}/${vv}/MERRA2_${vv}_${YEAR}.nc"
      if [ -e "${NATIVE_DIR}/${vv}/MERRA2_${vv}_${YEAR}.nc" ]; then
        rm `ls -1 ${TMP_DIR}/time_${vv}_*${YEAR}*`
      fi

    fi # if [ ! -e "${NATIVE_DIR}/${vv}/MERRA2_${vv}_${YEAR}.nc" ]; then

    # If using raijin
    MYHOST=`echo $HOST`
#    if [ ${MYHOST:0:6} == "raijin" ]; then
      echo "Regridding the data"
      cdo remapbil,${RUN_DIR}/targetgrid "${NATIVE_DIR}/${vv}/MERRA2_${vv}_${YEAR}.nc" "${REGRID_DIR}/${vv}/MERRA2_${vv}_${YEAR}.nc"
      # To convert to netcdf4-classic to work for climpact2 and EHF function
      ncks -7 "${REGRID_DIR}/${vv}/MERRA2_${vv}_${YEAR}.nc" "${REGRID_DIR}/${vv}/${vv}_${YEAR}.nc"
#    fi

    YEAR=$((${YEAR}+1))

  done # while [ ${YEAR} -lt $((${FIYEAR}+1)) ]; do

done # for vv in ${VARIABLES[@]}; do

